## @package FaceRecGUI
#  GUI class being executed.
from Tkinter import *
import sys
import training as train
from haar import faceCrop
import picamera
import time
import os
import os.path
import find_face
# suppress warnings
import warnings
warnings.filterwarnings('default', category=DeprecationWarning)

# initialize root window
root = Tk()
# initialize text field
tName = 'empty'

#Main window properties
root.title("Face-R")
#root.wm_iconbitmap('i/resourcespython.ico') # This does not work on the Pi.
root.minsize(width=550, height=200)

#Content

## Start face detection and recognition functions.
#
#  Display detected person as valid if the distance is not too high.
def startDetection():
   takePicture()
   foundPerson, foundDistance = find_face.startRecognition()
   if foundDistance < 1.7:
    T.insert(END, "\n" + foundPerson)
   else:
    T.insert(END, "\n No person found.")
   T.yview_pickplace("end") 

## Take reference picture.
#
#  Delete old reference picture if present.
#  Initialize Pi camera.
#  Preview camera picture for 5 seconds.
#  Take picture as jpg and convert it to grayscale pgm
#  Remove original jpg image.
def takePicture():
    if os.path.isfile('test_faces/reference.pgm'):
        os.remove('test_faces/reference.pgm')
    with picamera.PiCamera() as camera:
        camera.framerate = 24.0
        camera.resolution = (500, 400)
        camera.start_preview()
        time.sleep(5)
        camera.capture('test_faces/reference.jpg')
        camera.stop_preview()
        faceCrop('test_faces/reference.jpg',boxScale=1)
        os.remove('test_faces/reference.jpg')

but=Button(root, text="Start detection", command=startDetection, width=20, height=2)
but.grid(row=2, column=1)

#Textfield
S = Scrollbar(root)
T = Text(root, height=5, width=30)
S.grid(row=3, column=2)
T.grid(row=3, column=1)
S.config(command=T.yview)
T.config(yscrollcommand=S.set)
T.insert(END, "Ready.")

#camera/picture
photo = PhotoImage(file="resources/logo.pgm")
label = Label(root, image=photo, height=200, width=200)
label.grid(row=1, column=0, rowspan=3, padx=20, pady=20)

## Menu for user creation.
def giveName():
   global createWin
   createWin = Tk()
   createWin.title("New face")

   w = Label(createWin, text="Insert name", fg = "#1e3d7c",font = "Verdana 20 bold", pady=20)
   w.grid(row=0, columnspan=2)

   global tInput
   tInput = Text(createWin, height=1, width=30)
   tInput.grid(row=1, column=0)
   
   butN=Button(createWin, text="New user", command=newFace, width=20, height=2)
   butN.grid(row=1, column=1)

   createWin.mainloop

## Create new user
#
#  Calls methods for image creation and processing.
#  Stores new user in database.
def newFace():
   T.insert(END, "\n Processing images. Caution! This may take a while.")
   tName = tInput.get('1.0',END)
   print(tName)
   train.takePictures(tName)
   train.faceCrop('pictures/'+tName +'/' + '*.jpg',boxScale=1)
   T.insert(END, "\n User created.")
   createWin.destroy()

#Button1
but=Button(root, text="New face", command=giveName, width=20, height=2)
but.grid(row=1, column=1)
  
root.mainloop()
