## @package find_face
#  Provides functions for face recognition.

from sklearn.decomposition import RandomizedPCA
import numpy as np
import glob
import cv2
import math
import os.path
import string
import os
import matplotlib.pyplot as plt
import warnings
warnings.filterwarnings('default', category=DeprecationWarning)

## Gets ID from filename.
def ID_from_filename(filename):
    part = string.split(filename, '/')
    return part[1]
 
## Converts image to the right gray scale format
def prepare_image(filename):
    img_color = cv2.imread(filename)
    img_gray = cv2.cvtColor(img_color, cv2.cv.CV_RGB2GRAY)
    img_gray = cv2.equalizeHist(img_gray)
    return img_gray.flat

## Counts files in database.
def fcount(path, map = {}):
  count = 0
  for f in os.listdir(path):
    child = os.path.join(path, f)
    if os.path.isdir(child):
      child_count = fcount(child, map)
      count += child_count + 1 # unless include self
  map[path] = count
  return count

## Does the actual face recognition.
#
# Counts the number of pictures in database and loads them.
# Gets information from pictures and compares the distances 
# between several spots with the reference picture.
def startRecognition():
    IMG_RES = 92 * 112 # img resolution
    NUM_EIGENFACES = 5 # images per train person
    path = "pictures"
    map = {}
    print fcount(path, map)
    NUM_TRAINIMAGES = NUM_EIGENFACES * fcount(path, map) # total images in training set

    #loading training set from folder train_faces
    folders = glob.glob('pictures/*')
     
    # Create an array with flattened images X
    # and an array with ID of the people on each image y
    X = np.zeros([NUM_TRAINIMAGES, IMG_RES], dtype='int8')
    y = []

    # Populate training array with flattened imags from subfolders of train_faces and names
    c = 0
    for x, folder in enumerate(folders):
        train_faces = glob.glob(folder + '/*')
        for i, face in enumerate(train_faces):
            X[c,:] = prepare_image(face)
            y.append(ID_from_filename(face))
            c = c + 1

    # perform principal component analysis on the images
    pca = RandomizedPCA(n_components=NUM_EIGENFACES, whiten=True).fit(X)
    X_pca = pca.transform(X)

    # load test faces (usually one), located in folder test_faces
    test_faces = glob.glob('test_faces/*')

    if len(test_faces) > 0:

        # Create an array with flattened images X
        X = np.zeros([len(test_faces), IMG_RES], dtype='int8')
     
        # Populate test array with flattened imags from subfolders of train_faces
        for i, face in enumerate(test_faces):
            X[i,:] = prepare_image(face)
     
        # run through test images (usually one)
        for j, ref_pca in enumerate(pca.transform(X)):
            distances = []
            # Calculate euclidian distance from test image to each of the known images and save distances
            for i, test_pca in enumerate(X_pca):
                dist = math.sqrt(sum([diff**2 for diff in (ref_pca - test_pca)]))
                distances.append((dist, y[i]))
     
            found_ID = min(distances)[1]

            print "Identified (result: "+ str(found_ID) +" - dist - " + str(min(distances)[0])  + ")"
            return "found person: " + str(found_ID), min(distances)[0]

    else:  
        print "No face found."
        return "found person: " + str(0), 1000
        
